@@ -31,6 +31,7 @@
 #include "allocation.h"
 #include "loader.h"
 #include "log.h"
+#include "stack_allocation.h"
 
 #include <ctype.h>
 
@@ -479,6 +480,7 @@ VkResult loader_add_environment_layers(struct loader_instance *inst, const enum
                             // Only add it if it doesn't already appear in the layer list
                             if (!loader_find_layer_name_in_list(source_prop->info.layerName, target_list)) {
                                 if (0 == (source_prop->type_flags & VK_LAYER_TYPE_FLAG_META_LAYER)) {
+                                    source_prop->enabled_by_what = ENABLED_BY_WHAT_VK_INSTANCE_LAYERS;
                                     res = loader_add_layer_properties_to_list(inst, target_list, source_prop);
                                     if (res == VK_ERROR_OUT_OF_HOST_MEMORY) goto out;
                                     res = loader_add_layer_properties_to_list(inst, expanded_target_list, source_prop);
@@ -545,6 +547,7 @@ VkResult loader_add_environment_layers(struct loader_instance *inst, const enum
 
         // If not a meta-layer, simply add it.
         if (0 == (source_prop->type_flags & VK_LAYER_TYPE_FLAG_META_LAYER)) {
+            source_prop->enabled_by_what = ENABLED_BY_WHAT_VK_LOADER_LAYERS_ENABLE;
             res = loader_add_layer_properties_to_list(inst, target_list, source_prop);
             if (res == VK_ERROR_OUT_OF_HOST_MEMORY) goto out;
             res = loader_add_layer_properties_to_list(inst, expanded_target_list, source_prop);
