@@ -161,8 +161,9 @@ VkResult parse_layer_configurations(const struct loader_instance* inst, cJSON* s
     VkResult res = VK_SUCCESS;
 
     cJSON* layer_configurations = loader_cJSON_GetObjectItem(settings_object, "layers");
+    // If the layers object isn't present, return early with success to allow the settings file to still apply
     if (NULL == layer_configurations) {
-        return VK_ERROR_INITIALIZATION_FAILED;
+        return VK_SUCCESS;
     }
 
     uint32_t layer_configurations_count = loader_cJSON_GetArraySize(layer_configurations);
@@ -278,14 +279,13 @@ void log_settings(const struct loader_instance* inst, loader_settings* settings)
     loader_log(inst, VULKAN_LOADER_INFO_BIT, 0, "Using layer configurations found in loader settings from %s",
                settings->settings_file_path);
 
-    char cmd_line_msg[64];
+    char cmd_line_msg[64] = {0};
     size_t cmd_line_size = sizeof(cmd_line_msg);
-    size_t num_used = 0;
 
     cmd_line_msg[0] = '\0';
 
-    generate_debug_flag_str(settings->debug_level, cmd_line_size, cmd_line_msg, &num_used);
-    if (num_used > 0) {
+    generate_debug_flag_str(settings->debug_level, cmd_line_size, cmd_line_msg);
+    if (strlen(cmd_line_msg)) {
         loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Loader Settings Filters for Logging to Standard Error: %s", cmd_line_msg);
     }
 
@@ -484,7 +484,7 @@ out:
     return res;
 }
 
-VkResult update_global_loader_settings(void) {
+TEST_FUNCTION_EXPORT VkResult update_global_loader_settings(void) {
     loader_settings settings = {0};
     VkResult res = get_loader_settings(NULL, &settings);
     loader_platform_thread_lock_mutex(&global_loader_settings_lock);
@@ -538,8 +538,8 @@ void release_current_settings_lock(const struct loader_instance* inst) {
     }
 }
 
-VkResult get_settings_layers(const struct loader_instance* inst, struct loader_layer_list* settings_layers,
-                             bool* should_search_for_other_layers) {
+TEST_FUNCTION_EXPORT VkResult get_settings_layers(const struct loader_instance* inst, struct loader_layer_list* settings_layers,
+                                                  bool* should_search_for_other_layers) {
     VkResult res = VK_SUCCESS;
     *should_search_for_other_layers = true;  // default to true
 
